@@ -31,10 +31,6 @@
 #include <gtk/gtk.h>
 #include <gtk/gtkbutton.h>
 
-#include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-job.h>
-#include <libgnomeprintui/gnome-print-job-preview.h>
-
 #include <libsoup/soup.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -108,6 +104,7 @@ static GtkWidget *entry;
 static GtkWidget *popup_menu, *popup_menu_back, *popup_menu_forward, *popup_menu_home;
 static GtkWidget *toolbar_back, *toolbar_forward;
 static HTMLURL *baseURL = NULL;
+static GtkTooltips *tooltips;
 
 static GList *go_list;
 static int go_position;
@@ -127,44 +124,44 @@ static GnomeUIInfo file_menu [] = {
 
 static GnomeUIInfo test_menu[] = {
 	{ GNOME_APP_UI_ITEM, "Test 1", "Run test 1",
-	  test_cb, GINT_TO_POINTER (1), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (1), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 2", "Run test 2",
-	  test_cb, GINT_TO_POINTER (2), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (2), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 3", "Run test 3",
-	  test_cb, GINT_TO_POINTER (3), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (3), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 4", "Run test 4",
-	  test_cb, GINT_TO_POINTER (4), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (4), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 5", "Run test 5",
-	  test_cb, GINT_TO_POINTER (5), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (5), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 6", "Run test 6",
-	  test_cb, GINT_TO_POINTER (6), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (6), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 7", "Run test 7 (FreshMeat)",
-	  test_cb, GINT_TO_POINTER (7), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (7), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 8", "Run test 8 (local test)",
-	  test_cb, GINT_TO_POINTER (8), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (8), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 9", "Run test 9 (Form Test)",
-	  test_cb, GINT_TO_POINTER (9), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (9), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 10", "Run test 10 (Object Test)",
-	  test_cb, GINT_TO_POINTER (10), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (10), NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Test 11", "Run test 11 (Nowrap)",
-	  test_cb, GINT_TO_POINTER (11), NULL, 0, 0, 0, 0},
+	  test_cb, GINT_TO_POINTER (11), NULL, 0, NULL, 0, 0},
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo debug_menu[] = {
 	{ GNOME_APP_UI_ITEM, "Show bug list", "Show the layout bug list",
-	  bug_cb, NULL, NULL, 0, 0, 0, 0},
+	  bug_cb, NULL, NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Dump Object tree", "Dump Object tree to stdout",
-	  dump_cb, NULL, NULL, 0, 0, 0, 0},
+	  dump_cb, NULL, NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Dump Object tree (simple)", "Dump Simple Object tree to stdout",
-	  dump_simple_cb, NULL, NULL, 0, 0, 0, 0},
+	  dump_simple_cb, NULL, NULL, 0, NULL, 0, 0},
 	GNOMEUIINFO_TOGGLEITEM("Slow loading", "Load documents slowly", slow_cb, NULL),
 	{ GNOME_APP_UI_ITEM, "Force resize", "Force a resize event",
-	  resize_cb, NULL, NULL, 0, 0, 0 },
+	  resize_cb, NULL, NULL, 0, NULL, 0 },
 	{ GNOME_APP_UI_ITEM, "Force repaint", "Force a repaint event",
-	  redraw_cb, NULL, NULL, 0, 0, 0 },
+	  redraw_cb, NULL, NULL, 0, NULL, 0 },
 	{ GNOME_APP_UI_ITEM, "Select all", "Select all",
-	  select_all_cb, NULL, NULL, 0, 0, 0 },
+	  select_all_cb, NULL, NULL, 0, NULL, 0 },
 	GNOMEUIINFO_TOGGLEITEM ("Disable Animations", "Disable Animated Images",  animate_cb, NULL),
 	 
 	GNOMEUIINFO_END
@@ -172,11 +169,11 @@ static GnomeUIInfo debug_menu[] = {
 
 static GnomeUIInfo go_menu[] = {
 	{ GNOME_APP_UI_ITEM, "Back", "Return to the previous page in history list",
-	  back_cb, NULL, NULL, 0, 0, 0, 0},
+	  back_cb, NULL, NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Forward", "Go to the next page in history list", 
-	  forward_cb, NULL, NULL, 0, 0, 0, 0},
+	  forward_cb, NULL, NULL, 0, NULL, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Home", "Go to the homepage",
-	  home_cb, NULL, NULL, 0, 0, 0 },
+	  home_cb, NULL, NULL, 0, NULL, 0 },
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_END
 };
@@ -196,6 +193,7 @@ create_toolbars (GtkWidget *app)
 	GtkWidget *hbox;
 	GtkWidget *frame;
 	GtkWidget *toolbar;
+	GtkToolItem *item;
 	/* char *imgloc; */
 
 	dock = bonobo_dock_item_new ("testgtkhtml-toolbar1",
@@ -208,41 +206,39 @@ create_toolbars (GtkWidget *app)
 	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
 	gtk_box_pack_start (GTK_BOX (hbox), toolbar, FALSE, FALSE, 0);
 
-	toolbar_back = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-						NULL,
-						"Move back",
-						"Back",
-						gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_LARGE_TOOLBAR),
-						GTK_SIGNAL_FUNC (back_cb), NULL);
-	gtk_widget_set_sensitive(toolbar_back, FALSE);
-	toolbar_forward = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-						   NULL,
-						   "Move forward",
-						   "Forward",
-						   gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_LARGE_TOOLBAR),
-						   GTK_SIGNAL_FUNC (forward_cb), NULL);
-	gtk_widget_set_sensitive(toolbar_forward, FALSE);
-	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
-	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-				 NULL,
-				 "Stop loading",
-				 "Stop",
-				 gtk_image_new_from_stock ("gtk-stop", GTK_ICON_SIZE_LARGE_TOOLBAR),
-				 GTK_SIGNAL_FUNC (stop_cb),
-				 NULL);
-	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-				 NULL,
-				 "Reload page",
-				 "Reload",
-				 gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_LARGE_TOOLBAR),
-				 GTK_SIGNAL_FUNC (reload_cb), NULL);
-	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-				 NULL,
-				 "Home page",
-				 "Home",
-				 gtk_image_new_from_stock ("gtk-home", GTK_ICON_SIZE_LARGE_TOOLBAR),
-				 GTK_SIGNAL_FUNC (home_cb), NULL);
-	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+	item = gtk_tool_button_new_from_stock (GTK_STOCK_GO_BACK);
+	gtk_tool_item_set_tooltip (item, tooltips, "Move back", "Back");
+	g_signal_connect (item, "clicked", G_CALLBACK (back_cb), NULL);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+	gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
+	toolbar_back = GTK_WIDGET (item);
+
+	item = gtk_tool_button_new_from_stock (GTK_STOCK_GO_FORWARD);
+	gtk_tool_item_set_tooltip (item, tooltips, "Move forward", "Forward");
+	g_signal_connect (item, "clicked", G_CALLBACK (forward_cb), NULL);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+	gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
+	toolbar_forward = GTK_WIDGET (item);
+
+	item = gtk_separator_tool_item_new ();
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+	item = gtk_tool_button_new_from_stock (GTK_STOCK_STOP);
+	gtk_tool_item_set_tooltip (item, tooltips, "Stop loading", "Stop");
+	g_signal_connect (item, "clicked", G_CALLBACK (stop_cb), NULL);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+	item = gtk_tool_button_new_from_stock (GTK_STOCK_REFRESH);
+	gtk_tool_item_set_tooltip (item, tooltips, "Reload page", "Reload");
+	g_signal_connect (item, "clicked", G_CALLBACK (reload_cb), NULL);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+	item = gtk_tool_button_new_from_stock (GTK_STOCK_HOME);
+	gtk_tool_item_set_tooltip (item, tooltips, "Home page", "Home");
+	g_signal_connect (item, "clicked", G_CALLBACK (home_cb), NULL);
+
+	item = gtk_separator_tool_item_new ();
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
 
 	/* animator = gnome_animator_new_with_size (32, 32);
 
@@ -287,56 +283,85 @@ create_toolbars (GtkWidget *app)
 }
 
 static gint page_num, pages;
-static GnomeFont *font;
+static PangoLayout *layout;
 
 static void
-print_footer (GtkHTML *html, GnomePrintContext *context,
-	      gdouble x, gdouble y, gdouble width, gdouble height, gpointer user_data)
+print_footer (GtkHTML *html, GtkPrintContext *context, gdouble x, gdouble y,
+              gdouble width, gdouble height, gpointer user_data)
 {
-	gchar *text = g_strdup_printf ("- %d of %d -", page_num, pages);
-	gdouble tw = gnome_font_get_width_utf8 (font, "text");
+	gchar *text;
+	cairo_t *cr;
 
-	if (font) {
-		gnome_print_newpath     (context);
-		gnome_print_setrgbcolor (context, .0, .0, .0);
-		gnome_print_moveto      (context, x + (width - tw)/2, y - gnome_font_get_ascender (font));
-		gnome_print_setfont     (context, font);
-		gnome_print_show        (context, text);
-	}
+	text = g_strdup_printf ("- %d of %d -", page_num++, pages);
+
+	pango_layout_set_width (layout, width * PANGO_SCALE);
+	pango_layout_set_text (layout, text, -1);
+
+	cr = gtk_print_context_get_cairo_context (context);
+
+	cairo_save (cr);
+	cairo_move_to (cr, x, y);
+	pango_cairo_show_layout (cr, layout);
+	cairo_restore (cr);
 
 	g_free (text);
-	page_num++;
+}
+
+static void
+draw_page_cb (GtkPrintOperation *operation, GtkPrintContext *context,
+              gint page_nr, gpointer user_data)
+{
+	/* XXX GtkHTML's printing API doesn't really fit well with GtkPrint.
+	 *     Instead of calling a function for each page, GtkHTML prints
+	 *     everything in one shot. */
+
+	PangoFontDescription *desc;
+	PangoFontMetrics *metrics;
+	gdouble footer_height;
+
+	desc = pango_font_description_from_string ("Helvetica 12");
+
+	layout = gtk_print_context_create_pango_layout (context);
+	pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
+	pango_layout_set_font_description (layout, desc);
+
+	metrics = pango_context_get_metrics (
+		pango_layout_get_context (layout),
+		desc, pango_language_get_default ());
+	footer_height = (pango_font_metrics_get_ascent (metrics) +
+		pango_font_metrics_get_descent (metrics)) / PANGO_SCALE;
+	pango_font_metrics_unref (metrics);
+
+	pango_font_description_free (desc);
+
+	page_num = 1;
+	pages = gtk_html_print_page_get_pages_num (
+		html, context, .0, footer_height);
+
+	gtk_html_print_page_with_header_footer (
+		html, context, .0, footer_height,
+		NULL, print_footer, NULL);
+
+	g_object_unref (layout);
 }
 
 static void
 print_preview_cb (GtkWidget *widget,
 		  gpointer data)
 {
-	GnomePrintJob *print_master;
-	GnomePrintContext *print_context;
-	GtkWidget *preview;
+	GtkPrintOperation *operation;
 
-	print_master = gnome_print_job_new (NULL);
-	/* FIX2 gnome_print_master_set_paper (print_master, gnome_paper_with_name ("US-Letter")); */
-	gtk_html_print_set_master (html, print_master);
+	operation = gtk_print_operation_new ();
+	gtk_print_operation_set_n_pages (operation, 1);
 
-	print_context = gnome_print_job_get_context (print_master);
-	font = gnome_font_find_closest ("Helvetica", 12);
+	g_signal_connect (
+		operation, "draw-page",
+		G_CALLBACK (draw_page_cb), NULL);
 
-	page_num = 1;
-	pages = gtk_html_print_get_pages_num (html, print_context,
-					      .0, gnome_font_get_ascender (font) - gnome_font_get_descender (font));
-	gtk_html_print_with_header_footer (html, print_context,
-					   .0, gnome_font_get_ascender (font) - gnome_font_get_descender (font),
-					   NULL, print_footer, NULL);
-	if (font)
-		g_object_unref (font);
-	
-	gnome_print_job_close (print_master);
-	preview = gnome_print_job_preview_new (print_master, "Print Preview");
-	gtk_widget_show (preview);
+	gtk_print_operation_run (
+		operation, GTK_PRINT_OPERATION_ACTION_PREVIEW, NULL, NULL);
 
-	g_object_unref (print_master);
+	g_object_unref (operation);
 }
 
 static void
@@ -361,7 +386,7 @@ static void
 resize_cb (GtkWidget *widget, gpointer data)
 {
 	g_print ("forcing resize\n");
-	html_engine_calc_size (html->engine, FALSE);
+	html_engine_calc_size (html->engine, NULL);
 }
 
 static void
@@ -567,7 +592,7 @@ on_redirect (GtkHTML *html, const gchar *url, int delay, gpointer data) {
 
 		redirect_url = g_strdup(url);
 		
-		redirect_timerId = gtk_timeout_add (delay * 1000,(GtkFunction) redirect_timer_event, NULL);
+		redirect_timerId = g_timeout_add (delay * 1000,(GtkFunction) redirect_timer_event, NULL);
 	}
 }
 
@@ -636,7 +661,7 @@ object_requested_cmd (GtkHTML *html, GtkHTMLEmbedded *eb, void *data)
 		return FALSE;
 
 	gtk_widget_ref (GTK_WIDGET (eb));
-	gtk_timeout_add(rand() % 5000 + 1000, (GtkFunction) object_timeout, eb);
+	g_timeout_add(rand() % 5000 + 1000, (GtkFunction) object_timeout, eb);
 	/* object_timeout (eb); */
 
 	return TRUE;
@@ -835,7 +860,7 @@ goto_url(const char *url, int back_or_forward)
 
 	/* Remove any pending redirection */
 	if(redirect_timerId) {
-		gtk_timeout_remove(redirect_timerId);
+		g_source_remove(redirect_timerId);
 
 		redirect_timerId = 0;
 	}
@@ -1017,6 +1042,8 @@ main (gint argc, gchar *argv[])
 
 	g_signal_connect (app, "delete_event", G_CALLBACK (exit_cb), NULL);
 
+	tooltips = gtk_tooltips_new ();
+
 	create_toolbars (app);
 	bar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
 	gnome_app_set_statusbar (GNOME_APP (app), bar);
